/** @file config.h

	Skelton for retropc emulator

	@author Takeda.Toshiya
	@date   2006.08.18 -

	@note
	Modified for MBS1 by Sasaji at 2015.08.16

	@brief [ config ]
*/

#ifndef _CONFIG_H_
#define _CONFIG_H_

#include "common.h"
#include "vm/vm.h"

#define FILE_VERSION	0x30

#define IOPORT_USE_5FDD	(config.io_port & 0x01)
#define IOPORT_USE_3FDD	(config.io_port & 0x02)

void init_config();
void load_config(const _TCHAR *path);
void save_config();
void release_config();
bool get_number_in_path(_TCHAR *path, int *number);
bool set_number_in_path(_TCHAR *path, int number);

typedef struct pathex_st {
	_TCHAR path[_MAX_PATH];
	int    num;
} pathex_t;

#define MAX_HISTORY	8
#define UPDATE_HISTORY(path, recent, updated) { \
	int no = MAX_HISTORY - 1; \
	for(int i = 0; i < MAX_HISTORY; i++) { \
		if(_tcscmp(recent[i], path) == 0) { \
			no = i; \
			break; \
		} \
	} \
	for(int i = no; i > 0; i--) { \
		_tcscpy(recent[i], recent[i - 1]); \
	} \
	_tcscpy(recent[0], path); \
	if (no > 0) updated = true; \
}
#define UPDATE_HISTORY_EX(path, recent, updated) { \
	int no = MAX_HISTORY - 1; \
	for(int i = 0; i < MAX_HISTORY; i++) { \
		if(memcmp(&recent[i], &path, sizeof(path)) == 0) { \
			no = i; \
			break; \
		} \
	} \
	for(int i = no; i > 0; i--) { \
		memcpy(&recent[i], &recent[i - 1], sizeof(path)); \
	} \
	memcpy(&recent[0], &path, sizeof(path)); \
	if (no > 0) updated = true; \
}

typedef struct config_st {
	int version1;	// config file version
	int version2;

	// recent files
#ifdef USE_CART
	_TCHAR initial_cart_path[_MAX_PATH];
	_TCHAR recent_cart_path[MAX_HISTORY][_MAX_PATH];
#endif
#ifdef USE_FD1
	_TCHAR initial_disk_path[_MAX_PATH];
#if defined(USE_FD8) || defined(USE_FD7)
	pathex_t recent_disk_path[8][MAX_HISTORY];
	bool updated_recent_disk_path[8];
	pathex_t opened_disk_path[8];
#elif defined(USE_FD6) || defined(USE_FD5)
	pathex_t recent_disk_path[6][MAX_HISTORY];
	bool updated_recent_disk_path[6];
	pathex_t opened_disk_path[6];
#else
	pathex_t recent_disk_path[4][MAX_HISTORY];
	bool updated_recent_disk_path[4];
	pathex_t opened_disk_path[4];
#endif
#endif
#ifdef USE_QUICKDISK
	_TCHAR initial_quickdisk_path[_MAX_PATH];
	_TCHAR recent_quickdisk_path[MAX_HISTORY][_MAX_PATH];
#endif
#ifdef USE_DATAREC
	_TCHAR initial_datarec_path[_MAX_PATH];
	pathex_t recent_datarec_path[MAX_HISTORY];
	bool updated_recent_datarec_path;
	pathex_t opened_datarec_path;
#endif
#ifdef USE_MEDIA
	_TCHAR initial_media_path[_MAX_PATH];
	_TCHAR recent_media_path[MAX_HISTORY][_MAX_PATH];
#endif
#ifdef USE_BINARY_FILE1
	_TCHAR initial_binary_path[_MAX_PATH];
	_TCHAR recent_binary_path[2][MAX_HISTORY][_MAX_PATH];
#endif

	// screen
	int window_mode;
	bool stretch_screen;
	bool cutout_screen;

	// sound
	int sound_frequency;
	int sound_latency;

	// virtual machine
	int cpu_power;
#ifdef USE_FD1
	bool ignore_crc;
#endif
#ifdef USE_DIPSWITCH
	uint8 dipswitch;
#endif
#ifdef USE_BOOT_MODE
	int boot_mode;		// MZ-800, PASOPIA, PC-8801MA, PC-98DO
#endif
#ifdef USE_CPU_CLOCK_LOW
	bool cpu_clock_low;	// PC-8801MA, PC-9801E, PC-9801VM, PC-98DO
#endif
#if defined(_HC80) || defined(_PASOPIA) || defined(_PC8801MA)
	int device_type;
#endif
#if defined(USE_MONITOR_TYPE) || defined(USE_SCREEN_ROTATE)
	int monitor_type;
#endif
#ifdef _MBS1
	uint8 sys_mode;
	bool sync_irq;
#endif
#ifdef USE_SCANLINE
	bool scan_line;
#endif
#ifdef _MBS1
	int  afterimage;
	uint8 exram_size_num;
	bool mem_nowait;
	uint8 tvsuper;
	uint8 use_opengl;
	int8  disptmg_skew;
	int8  curdisp_skew;
	uint8 gl_filter_type;
	int volume;
	int fdd_type;

	// CONFIG_VERSION >= 0x05
	// bit0: use fdd5  bit1: use fdd3  bit2: use psg  bit3: use ex pia
	// bit4: use ex acia  bit5: use 9psg  bit6: use kanji
	int io_port;
	// bit0: show led  bit1: show msg  bit2: use joystick  bit3: inside led  bit4: enable lightpen bit5: enable mouse
	uint8 show_dlg;

	_TCHAR initial_printer_path[_MAX_PATH];
	_TCHAR initial_state_path[_MAX_PATH];
	_TCHAR saved_state_path[_MAX_PATH];

	// CONFIG_VERSION >= 0x06
	int fps_no;

	int disp_device_no;
	int screen_width;
	int screen_height;

	_TCHAR initial_autokey_path[_MAX_PATH];
	_TCHAR opened_autokey_path[_MAX_PATH];
	// support above params on cfg file
	_TCHAR msgboard_info_fontname[_MAX_PATH];
	_TCHAR msgboard_msg_fontname[_MAX_PATH];
	_TCHAR menu_fontname[_MAX_PATH];
	_TCHAR rom_path[_MAX_PATH];

	bool realmode_datarec;

	//
	uint8 screen_video_size;
	uint8 msgboard_info_fontsize;
	uint8 msgboard_msg_fontsize;
	uint8 menu_fontsize;

	// CONFIG_VERSION >= 0x07
	int beep_volume;
	int psg_volume;
	int psg6_volume;
	int psg9_volume;
	int relay_volume;
	int cmt_volume;
	int fdd_volume;
	bool mute;
	bool beep_mute;
	bool psg_mute;
	bool psg6_mute;
	bool psg9_mute;
	bool relay_mute;
	bool cmt_mute;
	bool fdd_mute;

	// CONFIG_VERSION >= 0x08
	_TCHAR printer_server_host[MAX_PRINTER][_MAX_PATH];
	int    printer_server_port[MAX_PRINTER];
	bool   printer_direct[MAX_PRINTER];

	int    comm_dipswitch[MAX_COMM];
	_TCHAR comm_server_host[MAX_COMM][_MAX_PATH];
	int    comm_server_port[MAX_COMM];
	bool   comm_server[MAX_COMM];
	bool   comm_connect[MAX_COMM];

	_TCHAR snapshot_path[_MAX_PATH];

	uint8 led_pos;

	_TCHAR font_path[_MAX_PATH];

	bool reckey_recording;
	bool reckey_playing;

	bool wav_reverse;
	bool wav_half;
	bool wav_correct;
	uint8 wav_correct_type;

	uint8 wav_sample_rate;
	uint8 wav_sample_bits;

#endif
#ifdef USE_SOUND_DEVICE_TYPE
	int sound_device_type;
#endif
} config_t;

extern config_t config;

#endif

