/** @file fifo.h

	Skelton for retropc emulator

	@author Takeda.Toshiya
	@date   2006.09.17-

	@brief [ fifo buffer ]
*/

#ifndef _FIFO_H_
#define _FIFO_H_

#include "common.h"

/**
	@brief fifo buffer
*/
class FIFO
{
private:
	int* buf;
	int cnt, rpt, wpt, size;
public:
	FIFO(int s) {
		cnt = rpt = wpt = 0;
		size = s;
		buf = (int*)malloc(s * sizeof(int));
	}
	void release() {
		free(buf);
	}
	void clear() {
		cnt = rpt = wpt = 0;
	}
	void write(int val) {
		if(cnt < size) {
			buf[wpt++] = val;
			if(wpt >= size) {
				wpt = 0;
			}
			cnt++;
		}
	}
	int read() {
		int val = 0;
		if(cnt) {
			val = buf[rpt++];
			if(rpt >= size) {
				rpt = 0;
			}
			cnt--;
		}
		return val;
	}
	int read_not_remove(int pt) {
		if(pt >= 0 && pt < cnt) {
			pt += rpt;
			if(pt >= size) {
				pt -= size;
			}
			return buf[pt];
		}
		return 0;
	}
	int count() {
		return cnt;
	}
	bool full() {
		return (cnt == size);
	}
	bool empty() {
		return (cnt == 0);
	}
};

#endif

