/** @file ag_config_dlg.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.04.08

	@brief [ ag_config_dlg ]
*/

#include "ag_config_dlg.h"
#include <agar/core/types.h>
#include "../gui.h"
//#include "../../config.h"
#include "ag_gui_base.h"
//#include "../../clocale.h"
#include "../../msgboard.h"

namespace GUI_AGAR
{

AG_CONFIG_DLG::AG_CONFIG_DLG(EMU *parent_emu, AG_GUI_BASE *parent_gui) : AG_DLG(parent_emu, parent_gui)
{
	memset(&param, 0, sizeof(param));
	param.fdd_type  = config.fdd_type;
	param.io_port = config.io_port;
	param.sys_mode_rev = (1 - (config.sys_mode & 1));

	param.wav_reverse = 0;
	param.wav_half = 1;
	param.wav_correct = 0;
	param.wav_correct_type = 0;

	param.wav_sample_rate = 3;
	param.wav_sample_bits = 0;

	param.disptmg_skew = config.disptmg_skew + 2;
	param.curdisp_skew = config.curdisp_skew + 2;

	param.gl_filter_type = config.gl_filter_type;
	param.led_pos = config.led_pos;

	param.exram_num = config.exram_size_num;

	paramtmp = NULL;

	dirdlg = new AG_DIR_DLG(parent_emu, parent_gui);
}

AG_CONFIG_DLG::~AG_CONFIG_DLG()
{
	delete dirdlg;
	delete paramtmp;
}

/**
 * create config dialog
 */
void AG_CONFIG_DLG::Create()
{
	param.disptmg_skew = config.disptmg_skew + 2;
	param.curdisp_skew = config.curdisp_skew + 2;
	param.use_opengl = config.use_opengl;
	param.gl_filter_type = config.gl_filter_type;
	param.led_pos = config.led_pos;

	param.fdd_type = emu->get_parami(EMU::ParamFddType);
	param.io_port = emu->get_parami(EMU::ParamIOPort);
	param.sys_mode_rev = (1 - (emu->get_parami(EMU::ParamSysMode) & 1));
	param.wav_reverse = config.wav_reverse ? 1 : 0;
	param.wav_half = config.wav_half ? 1 : 0;
	param.wav_correct = config.wav_correct ? 1 : 0;
	param.wav_correct_type = config.wav_correct_type;
	param.wav_sample_rate = config.wav_sample_rate;
	param.wav_sample_bits = config.wav_sample_bits;

	memcpy(param.snap_path, config.snapshot_path, _MAX_PATH);
	memcpy(param.rom_path, config.rom_path, _MAX_PATH);
	memcpy(param.font_path, config.font_path, _MAX_PATH);
	memcpy(param.msg_font_name, config.msgboard_msg_fontname, _MAX_PATH);
	sprintf(param.msg_font_size, "%d", config.msgboard_msg_fontsize);
	memcpy(param.info_font_name, config.msgboard_info_fontname, _MAX_PATH);
	sprintf(param.info_font_size, "%d", config.msgboard_info_fontsize);
	memcpy(param.menu_font_name, config.menu_fontname, _MAX_PATH);
	sprintf(param.menu_font_size, "%d", config.menu_fontsize);

	for(int i=0; i<MAX_PRINTER; i++) {
		memcpy(param.prn_host[i], config.printer_server_host[i], _MAX_PATH);
		sprintf(param.prn_port[i], "%d", config.printer_server_port[i]);
	}
	for(int i=0; i<MAX_COMM; i++) {
		memcpy(param.com_host[i], config.comm_server_host[i], _MAX_PATH);
		sprintf(param.com_port[i], "%d", config.comm_server_port[i]);
		param.com_dipsw[i] = config.comm_dipswitch[i];
	}

	param.exram_num = emu->get_parami(EMU::ParamExMemNum);

	if (paramtmp == NULL) {
		paramtmp = new ConfigDlgParam;
	}
	*paramtmp = param;

	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL | AG_WINDOW_DIALOG | AG_WINDOW_NORESIZE);
	AG_WindowSetCaptionS(win, _("Configure"));

	AG_Box *vbox_base;
	AG_Box *hbox_base;
	AG_Box *vbox_left;
	AG_Box *vbox_right;
	AG_Box *vbox;
	AG_Box *hbox;
	AG_Radio *rad;
	AG_Checkbox *cbox;
	AG_Notebook *nb;
	AG_NotebookTab *tabs[5];

	int i;
	char name[128];

	nb = AG_NotebookNew(win, AG_NOTEBOOK_HFILL);

	//
	// 0 Mode
	tabs[0] = AG_NotebookAddTab(nb, _("Mode"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[0], AG_BOX_HFILL | AG_BOX_VFILL);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	// left
	vbox_left = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

	// System Mode
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("System Mode (*)"));

	set_label(radSysModeBase[0],_("A Mode (S1)"));
	set_label(radSysModeBase[1],_("B Mode (L3)"));
	for(i=0; i<2; i++) {
		radSysMode[i] = radSysModeBase[i];
		if (i == (1 - (config.sys_mode & 1))) {
			radSysModeBase[i][0] = '>';
		}
	}
	radSysMode[2]=NULL;

	rad = AG_RadioNewInt(vbox, AG_RADIO_EXPAND, (const char **)radSysMode, &paramtmp->sys_mode_rev);

	// FDD type
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("FDD Type (*)"));

	set_label(radFddTypeBase[0],_("No FDD"));
	set_label(radFddTypeBase[1],_("3inch compact FDD (for L3)"));
	set_label(radFddTypeBase[2],_("5.25inch mini FDD (2D Type)"));
	set_label(radFddTypeBase[3],_("5.25inch mini FDD (2HD Type)"));
	for(i=0; i<4; i++) {
		radFddType[i] = radFddTypeBase[i];
		if (i == config.fdd_type) {
			radFddTypeBase[i][0] = '>';
		}
	}
	radFddType[4]=NULL;

	rad = AG_RadioNewInt(vbox, AG_RADIO_EXPAND, (const char **)radFddType, &paramtmp->fdd_type);
	AG_AddEvent(rad, "radio-changed", OnChangeFddType, "%Cp", this);

	// right
	vbox_right = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

	// I/O Port address
	vbox = AG_BoxNewVert(vbox_right, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("I/O Port Address (*)"));

	set_label(chkIOPortBase[0],_("5.25inch FDC  $FF00 - $FF14"));
	set_label(chkIOPortBase[1],_("3inch FDC (for L3)  $FF18 - $FF20"));
	set_label(chkIOPortBase[2],_("6voice PSG (for L3)  $FF30 - $FF33"));
	set_label(chkIOPortBase[3],_("Ex LPT Port  $FF3C - $FF3F"));
	set_label(chkIOPortBase[4],_("Ex COM Port  $FF40 - $FF41"));
	set_label(chkIOPortBase[5],_("9voice PSG (for L3)  $FF70 - $FF7F"));
	set_label(chkIOPortBase[6],_("KANJI ROM  $FF75 - $FF76"));
	for(i=0; i<7; i++) {
		if (config.io_port & (1 << i)) {
			chkIOPortBase[i][0] = '>';
		}
		cbox = AG_CheckboxNewFlag(vbox, 0, chkIOPortBase[i], (Uint *)&paramtmp->io_port, (1 << i));
		AG_AddEvent(cbox, "checkbox-changed", OnChangeIOPort, "%Cp %i", this, i);
		chkIOPort[i] = cbox;
	}
	chkIOPort[7] = NULL;

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, _("* Need restart program or \"PowerOn\"."));


	//
	// 1 screen
	tabs[1] = AG_NotebookAddTab(nb, _("Screen"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[1], AG_BOX_EXPAND);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

#ifdef USE_OPENGL
	// GL Filter Type
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("OpenGL"));

	const char *gl_use_list[] = {
		_("No"),
		_("Yes(sync)"),
		_("Yes(async)"),
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Use (*)"));
	UComboNew(hbox, gl_use_list, config.use_opengl, OnChangeUseOpenGL);

	const char *gl_filter_list[] = {
		_("Nearest Neighbour"),
		_("Linear"),
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Filter Type"));
	UComboNew(hbox, gl_filter_list, config.gl_filter_type, OnChangeGLFilterType);
#endif

	// CRTC
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("CRTC"));

	const char *skew_list[] = {
		"-2", "-1", "0", "1", "2", NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Disptmg Skew"));
	UComboNew(hbox, skew_list, config.disptmg_skew + 2, OnChangeDisptmgSkew);

	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Curdisp Skew (L3 Only)"));
	UComboNew(hbox, skew_list, config.curdisp_skew + 2, OnChangeCurdispSkew);

	// LED
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("LED"));

	const char *led_pos_list[] = {
		_("LeftTop"),
		_("RightTop"),
		_("LeftBottom"),
		_("RightBottom"),
		NULL
	};
	AG_LabelNewS(hbox, 0, _("Position"));
	UComboNew(hbox, led_pos_list, config.led_pos, OnChangeLedPos);

	// snapshot path
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Snapshot Path"), 40, paramtmp->snap_path, sizeof(paramtmp->snap_path)-1);
	AG_ButtonNewFn(hbox, 0, _("Folder..."), OnShowDirDlg, "%Cp %Cp %i", this, win, SnapShotPath);

	// font path
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Font Path"), 40, paramtmp->font_path, sizeof(paramtmp->font_path)-1);
	AG_ButtonNewFn(hbox, 0, _("Folder..."), OnShowDirDlg, "%Cp %Cp %i", this, win, FontPath);

	// msgboard font
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Message Font"), 16, paramtmp->msg_font_name, sizeof(paramtmp->msg_font_name)-1);
	TextboxNew(hbox, _(" Size"), 4, paramtmp->msg_font_size, sizeof(paramtmp->msg_font_size)-1);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Info Font"), 16, paramtmp->info_font_name, sizeof(paramtmp->info_font_name)-1);
	TextboxNew(hbox, _(" Size"), 4, paramtmp->info_font_size, sizeof(paramtmp->info_font_size)-1);

	// menu font
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Menu Font (*)"), 16, paramtmp->menu_font_name, sizeof(paramtmp->menu_font_name)-1);
	TextboxNew(hbox, _(" Size"), 4, paramtmp->menu_font_size, sizeof(paramtmp->menu_font_size)-1);

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, _("* Need restart program."));

	//
	// 2 tape
	tabs[2] = AG_NotebookAddTab(nb, _("Tape"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[2], AG_BOX_EXPAND);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	// Load wav file
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("Load Wav File"));

	AG_CheckboxNewInt(vbox, 0, _("Reverse Wave"), &paramtmp->wav_reverse);
	AG_CheckboxNewInt(vbox, 0, _("Half Wave"), &paramtmp->wav_half);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	cbox = AG_CheckboxNewInt(hbox, 0, _("Correct"), &paramtmp->wav_correct);

	const char *radCorrect[] = {
		_("COS Wave"),
		_("SIN Wave"),
		NULL
	};
	rad = AG_RadioNewInt(hbox, AG_RADIO_EXPAND, radCorrect, &paramtmp->wav_correct_type);

	AG_AddEvent(cbox, "checkbox-changed", OnChangeCorrect, "%Cp %Cp", this, rad);
//	if (paramtmp->wav_correct == 0) {
//		AG_WidgetDisable(rad);
//	} else {
//		AG_WidgetEnable(rad);
//	}

	// Save wav file
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("Save Wav File"));

	const char *rate_list[] = {
		"11025",
		"22050",
		"44100",
		"48000",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Sample Rate"));
	UComboNew(hbox, rate_list, paramtmp->wav_sample_rate, OnChangeSampleRate);

	const char *bits_list[] = {
		"8",
		"16",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Sample Bits"));
	UComboNew(hbox, bits_list, paramtmp->wav_sample_bits, OnChangeSampleBits);

	//
	// 3 network
	tabs[3] = AG_NotebookAddTab(nb, _("Network"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[3], AG_BOX_EXPAND);

	// LPT
	for(i=0; i<MAX_PRINTER; i++) {
		hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
		sprintf(name, _("LPT%d Hostname"), i);
		TextboxNew(hbox, name, 15, paramtmp->prn_host[i], sizeof(paramtmp->prn_host[i])-1);
		TextboxNew(hbox, _(" Port"), 6, paramtmp->prn_port[i], sizeof(paramtmp->prn_port[i])-1);
	}
	// COM
	const char *comm_baud_list[] = {
		_("S:300baud/F:1200baud"),
		_("S:600baud/F:2400baud"),
		_("S:1200baud/F:4800baud"),
		_("S:2400baud/F:9600baud"),
		NULL
	};
	for(i=0; i<MAX_COMM; i++) {
		hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
		sprintf(name, _("COM%d Hostname"), i);
		TextboxNew(hbox, name, 15, paramtmp->com_host[i], sizeof(paramtmp->com_host[i])-1);
		TextboxNew(hbox, _(" Port"), 6, paramtmp->com_port[i], sizeof(paramtmp->com_port[i])-1);

		UComboNew(hbox, comm_baud_list, paramtmp->com_dipsw[i]-1, OnChangeCommBaud, i);
	}

	//
	// 4 Memory
	tabs[4] = AG_NotebookAddTab(nb, _("Memory"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[4], AG_BOX_EXPAND);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("ROM Path"), 40, paramtmp->rom_path, sizeof(paramtmp->rom_path)-1);
	AG_ButtonNewFn(hbox, 0, _("Folder..."), OnShowDirDlg, "%Cp %Cp %i", this, win, RomPath);
	//
	const char *exram_list[] = {
		_("None"), _T("64KB"), _T("128KB"), _T("256KB"), _T("512KB"), NULL
	};
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Extended RAM"));
	UComboNew(hbox, exram_list, paramtmp->exram_num, OnChangeExtRam);
	if (0 <= config.exram_size_num && config.exram_size_num <= 4) {
		_TCHAR str[100];
		_tcscpy(str, _(" (Now "));
		_tcscat(str, exram_list[config.exram_size_num]);
		_tcscat(str, _T(")"));
		AG_LabelNewS(hbox, 0, str);
	}
	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, _("* Need restart program or \"PowerOn\"."));

	//
	// button
	hbox_base = AG_BoxNewHoriz(win, AG_BOX_EXPAND);
	AG_ButtonNewFn(hbox_base, 0, _("OK"), OnOk, "%Cp %Cp", this, win);
	AG_ButtonNewFn(hbox_base, 0, _("Cancel"), OnClose, "%Cp %Cp", this, win);

	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, win);

	gui->PostEtSystemPause(true);
	AG_WindowShow(win);

	//
	change_fdd_type(paramtmp->fdd_type);
}

void AG_CONFIG_DLG::change_fdd_type(int index)
{
	if (index == 0) {
		paramtmp->io_port = paramtmp->io_port & ~3;
		AG_WidgetEnable(chkIOPort[0]);
		AG_WidgetEnable(chkIOPort[1]);
	} else if (index == 1) {
		paramtmp->io_port = (paramtmp->io_port & ~1) | 2;
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
	} else if (index == 2 || index == 3) {
		paramtmp->io_port = (paramtmp->io_port & ~2) | 1;
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
	}
}

void AG_CONFIG_DLG::change_io_port(int index, int status)
{
	if (index == 0 && status & (1 << 0)) {
		paramtmp->io_port &= ~(1 << 1);
	} else if (index == 1 && status & (1 << 1)) {
		paramtmp->io_port &= ~(1 << 0);
	}
	if (index == 5 && status & (1 << 5)) {
		paramtmp->io_port &= ~(1 << 6);
	} else if (index == 6 && status & (1 << 6)) {
		paramtmp->io_port &= ~(1 << 5);
	}
}

void AG_CONFIG_DLG::set_label(char *label, const char *str)
{
	strcpy(label,"  ");
	strcat(label, str);
}

void AG_CONFIG_DLG::Close(AG_Window *win)
{
	AG_ObjectDetach(win);
	gui->PostEtSystemPause(false);
}

int AG_CONFIG_DLG::SetData(AG_Window *win)
{
	param = *paramtmp;

	config.disptmg_skew = param.disptmg_skew - 2;
	config.curdisp_skew = param.curdisp_skew - 2;
	config.use_opengl = (uint8)param.use_opengl;
	config.gl_filter_type = (uint8)param.gl_filter_type;
	config.led_pos = param.led_pos;

	emu->set_parami(EMU::ParamFddType, param.fdd_type);
	emu->set_parami(EMU::ParamIOPort, param.io_port);
	emu->set_parami(EMU::ParamSysMode, (1 - param.sys_mode_rev) | (config.sys_mode & ~1));

	config.wav_reverse = (param.wav_reverse == 1);
	config.wav_half = (param.wav_half == 1);
	config.wav_correct = ( param.wav_correct == 1);
	config.wav_correct_type = (uint8)param.wav_correct_type;
	config.wav_sample_rate = (uint8)param.wav_sample_rate;
	config.wav_sample_bits = (uint8)param.wav_sample_bits;

	int val = 0;
	for(int i=0; i<MAX_PRINTER; i++) {
		strcpy(config.printer_server_host[i], param.prn_host[i]);
		val = 0;
		if (sscanf(param.prn_port[i], "%d", &val) == 1 && val > 0 && val < 65535) {
			config.printer_server_port[i] = val;
		}
	}
	for(int i=0; i<MAX_COMM; i++) {
		strcpy(config.comm_server_host[i], param.com_host[i]);
		val = 0;
		if (sscanf(param.com_port[i], "%d", &val) == 1 && val > 0 && val < 65535) {
			config.comm_server_port[i] = val;
		}
		config.comm_dipswitch[i] = param.com_dipsw[i];
	}

	strcpy(config.snapshot_path, param.snap_path);
	UTILITY::add_path_separator(config.snapshot_path);
	strcpy(config.font_path, param.font_path);
	UTILITY::add_path_separator(config.font_path);
	strcpy(config.msgboard_msg_fontname, param.msg_font_name);
	strcpy(config.msgboard_info_fontname, param.info_font_name);
	val = 0;
	if (sscanf(param.msg_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_msg_fontsize = val;
	}
	val = 0;
	if (sscanf(param.info_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_info_fontsize = val;
	}
	strcpy(config.menu_fontname, param.menu_font_name);
	val = 0;
	if (sscanf(param.menu_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.menu_fontsize = val;
	}

	strcpy(config.rom_path, param.rom_path);
	UTILITY::add_path_separator(config.rom_path);

	emu->set_parami(EMU::ParamExMemNum, param.exram_num);

	// set message font
	MsgBoard *msgboard = emu->get_msgboard();
	if (msgboard) {
		msgboard->SetFont();
	}
	// set menu font
//	gui->ConfigLoad();

	save_config();
	emu->change_pos_led(config.led_pos);
#ifdef USE_OPENGL
	emu->change_opengl_attr();
#endif

	delete paramtmp;
	paramtmp = NULL;

	return 1;
}

void AG_CONFIG_DLG::show_dir_dlg(AG_Window *win, DirDlgType type)
{
	switch(type) {
	case RomPath:
		dirdlg->CreateLoad(_("Rom Path"), paramtmp->rom_path, sizeof(paramtmp->rom_path), win);
		break;
	case SnapShotPath:
		dirdlg->CreateLoad(_("Snapshot Path"), paramtmp->snap_path, sizeof(paramtmp->snap_path), win);
		break;
	case FontPath:
		dirdlg->CreateLoad(_("Font Path"), paramtmp->font_path, sizeof(paramtmp->font_path), win);
		break;
	}
}

/*
 * Event Handler (static)
 */
void AG_CONFIG_DLG::OnOk(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	if (dlg->SetData(win)) {
		dlg->Close(win);
	}
}

void AG_CONFIG_DLG::OnClose(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	dlg->Close(win);
}

void AG_CONFIG_DLG::OnChangeCorrect(AG_Event *event)
{
//	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
//	AG_Radio *rad = (AG_Radio *)AG_PTR(2);
//
//	if (dlg->paramtmp->wav_correct == 0) {
//		AG_WidgetDisable(rad);
//	} else {
//		AG_WidgetEnable(rad);
//	}
}

void AG_CONFIG_DLG::OnChangeSampleRate(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->wav_sample_rate = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeSampleBits(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->wav_sample_bits = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeFddType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = (int)AG_INT(2);

	dlg->change_fdd_type(index);
}

void AG_CONFIG_DLG::OnChangeIOPort(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = AG_INT(2);
	int status = AG_INT(3);

	dlg->change_io_port(index, status);
}

void AG_CONFIG_DLG::OnChangeDisptmgSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->disptmg_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCurdispSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->curdisp_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeUseOpenGL(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->use_opengl = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeGLFilterType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->gl_filter_type = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeLedPos(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->led_pos = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCommBaud(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int num   = AG_INT(2);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(3);
	int state = AG_INT(4);

	if (state) {
		dlg->paramtmp->com_dipsw[num] = item->label + 1;
	}
}

void AG_CONFIG_DLG::OnChangeExtRam(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->exram_num = item->label;
	}
}

void AG_CONFIG_DLG::OnShowDirDlg(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);
	DirDlgType type = (DirDlgType)AG_INT(3);

	dlg->show_dir_dlg(win, type);
}

}; /* namespace GUI_AGAR */
