/** @file ag_gui.cpp

	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.3.3

	@brief [ ag_gui for mbs1 ]
*/

#include "ag_gui.h"
#include "ag_file_dlg.h"
#include "ag_config_dlg.h"
#include "ag_keybind_dlg.h"
#include "../../clocale.h"
#include "../../sdl_main.h"
#if defined(USE_LIB_GTK)
#include <gtk/gtk.h>
#endif

namespace GUI_AGAR
{

/****************************************

 GUI class using AGAR

 ****************************************/

AG_GUI::AG_GUI(int argc, char **argv, EMU *new_emu) : AG_GUI_BASE(argc, argv, new_emu)
{
#if defined(USE_LIB_GTK)
	gtk_init(&argc, &argv);
#endif
	configbox = NULL;
	keybindbox = NULL;
}

AG_GUI::~AG_GUI()
{
	delete keybindbox;
	delete configbox;
}

int AG_GUI::Init()
{
	int rc = 0;

	rc = AG_GUI_BASE::Init();
	if (rc < 0) return rc;

	// dialog
	configbox = new AG_CONFIG_DLG(emu, this);
	keybindbox = new AG_KEYBIND_DLG(emu, this);

	return rc;
}

AG_DLG *AG_GUI::GetDlgPtr(int id)
{
	AG_DLG *p = NULL;

	switch(id) {
		case ID_KEYBINDBOX:
			p = keybindbox;
			break;
		default:
			p = configbox;
			break;
	}
	return p;
}

/**
 * create menu bar
 */
void AG_GUI::set_menu_item(AG_Menu *menu)
{
	AG_MenuItem *mt_control;
	AG_MenuItem *mt_tape;
	AG_MenuItem *mt_fdd;
	AG_MenuItem *mt_screen;
	AG_MenuItem *mt_sound;
	AG_MenuItem *mt_devices;
	AG_MenuItem *mt_options;
	AG_MenuItem *mt_help;

	AG_MenuItem *mi;
	AG_MenuItem *ms;

	char name[128];

	// create menu
	mt_control = AG_MenuNode(menu->root, _("Control"), NULL);
	{
		//                                **--------------- ------
		mi = AG_MenuAction(mt_control, menu_str(_("PowerOn\tAlt+F3")), NULL, OnSelectReset, "%Cp", this);
		mi = AG_MenuAction(mt_control, menu_str(_("Reset Switch\tAlt+R")), NULL, OnSelectWarmReset, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuNode(mt_control,   menu_str(_("System Mode")), NULL);
			ms = AG_MenuAction(mi,     menu_str(_("A Mode (S1)\tAlt+M")), NULL, OnSelectSystemMode, "%Cp %i", this, 1);
				 AG_MenuSetPollFn(ms, OnUpdateSystemMode, "%Cp %i", this, 1);
			ms = AG_MenuAction(mi,     menu_str(_("B Mode (L3)\tAlt+M")), NULL, OnSelectSystemMode, "%Cp %i", this, 0);
				 AG_MenuSetPollFn(ms, OnUpdateSystemMode, "%Cp %i", this, 0);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuNode(mt_control,   menu_str(_("FDD Type")), NULL);
			ms = AG_MenuAction(mi,     menu_str(_("No FDD\tAlt+F")), NULL, OnSelectFDDType, "%Cp %i", this, 0);
				 AG_MenuSetPollFn(ms, OnUpdateFDDType, "%Cp %i", this, 0);
			ms = AG_MenuAction(mi,     menu_str(_("3inch compact FDD\tAlt+F")), NULL, OnSelectFDDType, "%Cp %i", this, 1);
				 AG_MenuSetPollFn(ms, OnUpdateFDDType, "%Cp %i", this, 1);
			ms = AG_MenuAction(mi,     menu_str(_("5.25inch mini FDD (2D Type)\tAlt+F")), NULL, OnSelectFDDType, "%Cp %i", this, 2);
				 AG_MenuSetPollFn(ms, OnUpdateFDDType, "%Cp %i", this, 2);
			ms = AG_MenuAction(mi,     menu_str(_("5.25inch mini FDD (2HD Type)\tAlt+F")), NULL, OnSelectFDDType, "%Cp %i", this, 3);
				 AG_MenuSetPollFn(ms, OnUpdateFDDType, "%Cp %i", this, 3);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, menu_str(_("Pause\tAlt+Q")), NULL, OnSelectPause, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdatePause, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuNode(mt_control,   menu_str(_("CPU Speed")), NULL);
			ms = AG_MenuAction(mi,     menu_str(_("CPU x0.5\tAlt+9")), NULL, OnSelectCPUPower, "%Cp %i", this, 0);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 0);
			ms = AG_MenuAction(mi,     menu_str(_("CPU x1\tAlt+1")), NULL, OnSelectCPUPower, "%Cp %i", this, 1);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 1);
			ms = AG_MenuAction(mi,     menu_str(_("CPU x2\tAlt+2")), NULL, OnSelectCPUPower, "%Cp %i", this, 2);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 2);
			ms = AG_MenuAction(mi,     menu_str(_("CPU x4\tAlt+3")), NULL, OnSelectCPUPower, "%Cp %i", this, 3);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 3);
			ms = AG_MenuAction(mi,     menu_str(_("CPU x8\tAlt+4")), NULL, OnSelectCPUPower, "%Cp %i", this, 4);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 4);
			ms = AG_MenuAction(mi,     menu_str(_("CPU x16\tAlt+5")), NULL, OnSelectCPUPower, "%Cp %i", this, 5);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 5);
			AG_MenuSeparator(mi);
			ms = AG_MenuAction(mi, menu_str(_("Sync With CPU Speed\tAlt+0")), NULL, OnSelectSyncIRQ, "%Cp", this);
				AG_MenuSetPollFn(ms, OnUpdateSyncIRQ, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, menu_str(_("Memory No Wait")), NULL, OnSelectMemNoWait, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateMemNoWait, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuNode(mt_control,   menu_str(_("Auto Key")), NULL);
			ms = AG_MenuAction(mi,     menu_str(_("Open...")), NULL, OnSelectOpenAutoKey, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateOpenAutoKey, "%Cp", this);
			ms = AG_MenuAction(mi,     menu_str(_("Paste")), NULL, OnSelectStartAutoKey, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateStartAutoKey, "%Cp", this);
			 ms = AG_MenuAction(mi    , menu_str(_("Stop")), NULL, OnSelectStopAutoKey, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuNode(mt_control,   menu_str(_("Record Key")), NULL);
			ms = AG_MenuAction(mi,     menu_str(_("Play...")), NULL, OnSelectPlayRecKey, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdatePlayRecKey, "%Cp", this);
			ms = AG_MenuAction(mi    , menu_str(_("Stop Playing")), NULL, OnSelectStopPlayRecKey, "%Cp", this);
			AG_MenuSeparator(mi);
			ms = AG_MenuAction(mi,     menu_str(_("Record...")), NULL, OnSelectRecordRecKey, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateRecordRecKey, "%Cp", this);
			ms = AG_MenuAction(mi    , menu_str(_("Stop Recording")), NULL, OnSelectStopRecordRecKey, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, menu_str(_("Load State...\tAlt+O")), NULL, OnSelectLoadState, "%Cp", this);
		mi = AG_MenuAction(mt_control, menu_str(_("Save State...")), NULL, OnSelectSaveState, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, menu_str(_("Exit\tAlt+F4")), NULL, OnSelectExit, "%Cp", this);
	}
	mt_tape = AG_MenuNode(menu->root, _("Tape"), NULL);
	{
		//                             **--------------- ------
		mi = AG_MenuAction(mt_tape, menu_str(_("Play...\tAlt+F7")), NULL, OnSelectLoadDataRec, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateLoadDataRec, "%Cp", this);
		mi = AG_MenuAction(mt_tape, menu_str(_("Record...\tAlt+F6")), NULL, OnSelectSaveDataRec, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateSaveDataRec, "%Cp", this);
		mi = AG_MenuAction(mt_tape, menu_str(_("Rewind\tAlt+F5")), NULL, OnSelectRewindDataRec, "%Cp", this);
		mi = AG_MenuAction(mt_tape, menu_str(_("F.F.\tAlt+F8")), NULL, OnSelectFastForwardDataRec, "%Cp", this);
		mi = AG_MenuAction(mt_tape, menu_str(_("Eject")), NULL, OnSelectCloseDataRec, "%Cp", this);
		AG_MenuSeparator(mt_tape);
		mi = AG_MenuAction(mt_tape, menu_str(_("Real Mode")), NULL, OnSelectRealModeDataRec, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateRealModeDataRec, "%Cp", this);
		AG_MenuSeparator(mt_tape);
		mi = AG_MenuDynamicItem(mt_tape, menu_str(_("Recent Files")), NULL, OnUpdateRecent, "%Cp %i %i", this, AG_FILE_DLG::DATAREC, 0);
	}
	for(int drv=0; drv<USE_DRIVE; drv++) {
		sprintf(name,_("FDD%d"),drv);
		mt_fdd = AG_MenuNode(menu->root, name, NULL);
		{
		//                                **--------------- ------
			sprintf(name,              menu_str(_("Open...\tAlt+F%d")), drv + 9);
			mi = AG_MenuAction(mt_fdd, name, NULL, OnSelectOpenFloppyDisk, "%Cp %i", this, drv);
				 AG_MenuSetPollFn(mi, OnUpdateOpenFloppyDisk, "%Cp %i", this, drv);
			mi = AG_MenuAction(mt_fdd, menu_str(_("Change Side to A")), NULL, OnSelectChangeSideFloppyDisk, "%Cp %i", this, drv);
				 AG_MenuSetPollFn(mi, OnUpdateChangeSideFloppyDisk, "%Cp %i", this, drv);
			mi = AG_MenuAction(mt_fdd, menu_str(_("Close")), NULL, OnSelectCloseFloppyDisk, "%Cp %i", this, drv);
			AG_MenuSeparator(mt_fdd);
			mi = AG_MenuAction(mt_fdd, menu_str(_("Write Protect")), NULL, OnSelectWriteProtectFloppyDisk, "%Cp %i", this, drv);
				 AG_MenuSetPollFn(mi, OnUpdateWriteProtectFloppyDisk, "%Cp %i", this, drv);
			AG_MenuSeparator(mt_fdd);
			mi = AG_MenuDynamicItem(mt_fdd, menu_str(_("Multi Volume")), NULL, OnUpdateMultiVolumeList, "%Cp %i", this, drv);
			AG_MenuSeparator(mt_fdd);
			mi = AG_MenuDynamicItem(mt_fdd, menu_str(_("Recent Files")), NULL, OnUpdateRecent, "%Cp %i %i", this, AG_FILE_DLG::FLOPPY, drv);
		}
	}
	mt_screen = AG_MenuNode(menu->root, _("Screen"), NULL);
	{
		//                               **--------------- ------
		mi = AG_MenuNode(mt_screen,   menu_str(_("Frame Rate")), NULL);
			ms = AG_MenuAction(mi,    menu_str(_("Auto")), NULL, OnSelectFrameRate, "%Cp %i", this, -1);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, -1);
			ms = AG_MenuAction(mi,    menu_str(_("60fps")), NULL, OnSelectFrameRate, "%Cp %i", this, 0);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 0);
			ms = AG_MenuAction(mi,    menu_str(_("30fps")), NULL, OnSelectFrameRate, "%Cp %i", this, 1);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 1);
			ms = AG_MenuAction(mi,    menu_str(_("20fps")), NULL, OnSelectFrameRate, "%Cp %i", this, 2);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 2);
			ms = AG_MenuAction(mi,    menu_str(_("15fps")), NULL, OnSelectFrameRate, "%Cp %i", this, 3);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 3);
			ms = AG_MenuAction(mi,    menu_str(_("12fps")), NULL, OnSelectFrameRate, "%Cp %i", this, 4);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 4);
			ms = AG_MenuAction(mi,    menu_str(_("10fps")), NULL, OnSelectFrameRate, "%Cp %i", this, 5);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 5);
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuNode(mt_screen,   menu_str(_("Record Screen")), NULL);
		{
			for(int i = 0; i < 2; i++) {
				strcpy(name, "  ");
				GetRecordVideoSizeStr(i, &name[2]);
				ms = AG_MenuAction(mi, name, NULL, OnSelectScreenRecordSize, "%Cp %i", this, i);
					 AG_MenuSetPollFn(ms, OnUpdateScreenRecordSize, "%Cp %i", this, i);
			}
			AG_MenuSeparator(mi);
#ifdef USE_REC_VIDEO
			ms = AG_MenuAction(mi,    menu_str(_("Rec 60fps")), NULL, OnSelectScreenRecordFrameRate, "%Cp %i", this, 0);
			AG_MenuSetPollFn(ms, OnUpdateScreenRecordFrameRate, "%Cp %i", this, 0);
			ms = AG_MenuAction(mi,    menu_str(_("Rec 30fps")), NULL, OnSelectScreenRecordFrameRate, "%Cp %i", this, 1);
			AG_MenuSetPollFn(ms, OnUpdateScreenRecordFrameRate, "%Cp %i", this, 1);
			ms = AG_MenuAction(mi,    menu_str(_("Rec 20fps")), NULL, OnSelectScreenRecordFrameRate, "%Cp %i", this, 2);
			AG_MenuSetPollFn(ms, OnUpdateScreenRecordFrameRate, "%Cp %i", this, 2);
			ms = AG_MenuAction(mi,    menu_str(_("Rec 15fps")), NULL, OnSelectScreenRecordFrameRate, "%Cp %i", this, 3);
			AG_MenuSetPollFn(ms, OnUpdateScreenRecordFrameRate, "%Cp %i", this, 3);
			ms = AG_MenuAction(mi,    menu_str(_("Rec 12fps")), NULL, OnSelectScreenRecordFrameRate, "%Cp %i", this, 4);
			AG_MenuSetPollFn(ms, OnUpdateScreenRecordFrameRate, "%Cp %i", this, 4);
			ms = AG_MenuAction(mi,    menu_str(_("Rec 10fps")), NULL, OnSelectScreenRecordFrameRate, "%Cp %i", this, 5);
			AG_MenuSetPollFn(ms, OnUpdateScreenRecordFrameRate, "%Cp %i", this, 5);
			ms = AG_MenuAction(mi,    menu_str(_("Stop")), NULL, OnSelectStopScreenRecord, "%Cp", this);
//			AG_MenuSetPollFn(ms, OnUpdateStopScreenRecord, "%Cp", this);
			AG_MenuSeparator(mi);
#endif
			ms = AG_MenuAction(mi,    menu_str(_("Capture")), NULL, OnSelectScreenCapture, "%Cp", this);
		}
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuNode(mt_screen,   menu_str(_("Window")), NULL);
		for(int i = 0; i < emu->get_window_mode_count(); i++) {
			strcpy(name, "  ");
			GetWindowModeStr(i, &name[2]);
			ms = AG_MenuAction(mi, name, NULL, OnSelectWindowMode, "%Cp %i", this, i);
				 AG_MenuSetPollFn(ms, OnUpdateWindowMode, "%Cp %i", this, i);
		}
		mi = AG_MenuNode(mt_screen,   menu_str(_("Fullscreen")), NULL);
			ms = AG_MenuAction(mi, menu_str(_("Stretch Screen")), NULL, OnSelectStretchScreen, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateStretchScreen, "%Cp", this);
			ms = AG_MenuAction(mi, menu_str(_("Cutout Screen")), NULL, OnSelectCutoutScreen, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateCutoutScreen, "%Cp", this);
			AG_MenuSeparator(mi);
		for(int i = 0; i < emu->get_screen_mode_count(); i++) {
			strcpy(name, "  ");
			GetFullScreenModeStr(i, &name[2]);
			ms = AG_MenuAction(mi, name, NULL, OnSelectScreenMode, "%Cp %i", this, i);
				 AG_MenuSetPollFn(ms, OnUpdateScreenMode, "%Cp %i", this, i);
		}
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuAction(mt_screen, menu_str(_("Scanline\tAlt+S")), NULL, OnSelectScanLine, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateScanLine, "%Cp", this);
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuAction(mt_screen, menu_str(_("Afterimage1\tAlt+T")), NULL, OnSelectAfterImage, "%Cp %i", this, 1);
			 AG_MenuSetPollFn(mi, OnUpdateAfterImage, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_screen, menu_str(_("Afterimage2\tAlt+T")), NULL, OnSelectAfterImage, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateAfterImage, "%Cp %i", this, 2);
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuAction(mt_screen, menu_str(_("Digital RGB")), NULL, OnSelectRGBType, "%Cp %i", this, 0);
			 AG_MenuSetPollFn(mi, OnUpdateRGBType, "%Cp %i", this, 0);
		mi = AG_MenuAction(mt_screen, menu_str(_("Analog RGB")), NULL, OnSelectRGBType, "%Cp %i", this, 1);
			 AG_MenuSetPollFn(mi, OnUpdateRGBType, "%Cp %i", this, 1);
#ifdef USE_OPENGL
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuAction(mt_screen, menu_str(_("Use OpenGL(Sync)\tAlt+Y")), NULL, OnSelectUseOpenGL, "%Cp %i", this, 1);
			 AG_MenuSetPollFn(mi, OnUpdateUseOpenGL, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_screen, menu_str(_("Use OpenGL(Async)\tAlt+Y")), NULL, OnSelectUseOpenGL, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateUseOpenGL, "%Cp %i", this, 2);
#endif
	}
	mt_sound = AG_MenuNode(menu->root, _("Sound"), NULL);
	{
		//                              **--------------- ------
		mi = AG_MenuAction(mt_sound, menu_str(_("Volume...\tAlt+V")), NULL, OnSelectSoundVolume, "%Cp", this);
		AG_MenuSeparator(mt_sound);
		mi = AG_MenuNode(mt_sound,   menu_str(_("Record Sound")), NULL);
		{
			ms = AG_MenuAction(mi, menu_str(_("Start Recording")), NULL, OnSelectSoundStartRecord, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateSoundStartRecord, "%Cp", this);
			ms = AG_MenuAction(mi, menu_str(_("Stop Recording")), NULL, OnSelectSoundStopRecord, "%Cp", this);
//				 AG_MenuSetPollFn(ms, OnUpdateSoundStopRecord, "%Cp", this);
		}
		AG_MenuSeparator(mt_sound);
//		mi = AG_MenuAction(mt_sound, menu_str(_("2000Hz")), NULL, OnSelectSoundRate, "%Cp %i", this, 0);
//			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 0);
//		mi = AG_MenuAction(mt_sound, menu_str(_("4000Hz")), NULL, OnSelectSoundRate, "%Cp %i", this, 1);
//			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_sound, menu_str(_("8000Hz")), NULL, OnSelectSoundRate, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 2);
		mi = AG_MenuAction(mt_sound, menu_str(_("11025Hz")), NULL, OnSelectSoundRate, "%Cp %i", this, 3);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 3);
		mi = AG_MenuAction(mt_sound, menu_str(_("22050Hz")), NULL, OnSelectSoundRate, "%Cp %i", this, 4);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 4);
		mi = AG_MenuAction(mt_sound, menu_str(_("44100Hz")), NULL, OnSelectSoundRate, "%Cp %i", this, 5);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 5);
		mi = AG_MenuAction(mt_sound, menu_str(_("48000Hz")), NULL, OnSelectSoundRate, "%Cp %i", this, 6);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 6);
		mi = AG_MenuAction(mt_sound, menu_str(_("96000Hz")), NULL, OnSelectSoundRate, "%Cp %i", this, 7);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 7);
		AG_MenuSeparator(mt_sound);
		mi = AG_MenuAction(mt_sound, menu_str(_("50msec")), NULL, OnSelectSoundLatency, "%Cp %i", this, 0);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 0);
		mi = AG_MenuAction(mt_sound, menu_str(_("100msec")), NULL, OnSelectSoundLatency, "%Cp %i", this, 1);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_sound, menu_str(_("200msec")), NULL, OnSelectSoundLatency, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 2);
		mi = AG_MenuAction(mt_sound, menu_str(_("300msec")), NULL, OnSelectSoundLatency, "%Cp %i", this, 3);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 3);
		mi = AG_MenuAction(mt_sound, menu_str(_("400msec")), NULL, OnSelectSoundLatency, "%Cp %i", this, 4);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 4);
	}
	mt_devices = AG_MenuNode(menu->root, _("Devices"), NULL);
	{
		for(int drv = 0; drv < MAX_PRINTER; drv++) {
			sprintf(name,menu_str(_("LPT%d")), drv);
			mi = AG_MenuNode(mt_devices, name, NULL);
			{
				ms = AG_MenuAction(mi, menu_str(_("Save...")), NULL, OnSelectSavePrinter, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdateSavePrinter, "%Cp %i", this, drv);
				ms = AG_MenuAction(mi, menu_str(_("Print to mpprinter")), NULL, OnSelectPrintPrinter, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdatePrintPrinter, "%Cp %i", this, drv);
				ms = AG_MenuAction(mi, menu_str(_("Clear")), NULL, OnSelectClearPrinter, "%Cp %i", this, drv);
//					 AG_MenuSetPollFn(ms, OnUpdateClearPrinter, "%Cp %i", this, drv);
				AG_MenuSeparator(mi);
				ms = AG_MenuAction(mi, menu_str(_("Direct Send to mpprinter")), NULL, OnSelectDirectPrinter, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdateDirectPrinter, "%Cp %i", this, drv);
			}
		}
		AG_MenuSeparator(mt_devices);
		for(int drv = 0; drv < MAX_COMM; drv++) {
			sprintf(name,menu_str(_("COM%d")), drv);
			mi = AG_MenuNode(mt_devices, name, NULL);
			{
				ms = AG_MenuAction(mi, menu_str(_("Enable Server")), NULL, OnSelectCommServer, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdateCommServer, "%Cp %i", this, drv);
				ms = AG_MenuAction(mi, menu_str(_("Connect")), NULL, OnSelectCommConnect, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdateCommConnect, "%Cp %i", this, drv);
			}
		}
	}
	mt_options = AG_MenuNode(menu->root, _("Options"), NULL);
	{
		//                                **--------------- ------
		mi = AG_MenuAction(mt_options, menu_str(_("Show LED\tAlt+L")), NULL, OnSelectLedBox, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateLedBox, "%Cp", this);
		mi = AG_MenuAction(mt_options, menu_str(_("Show Message\tAlt+Z")), NULL, OnSelectMsgBoard, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateMsgBoard, "%Cp", this);
		AG_MenuSeparator(mt_options);
		mi = AG_MenuAction(mt_options, menu_str(_("Use Joypad")), NULL, OnSelectUseJoypad, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateUseJoypad, "%Cp", this);
		mi = AG_MenuAction(mt_options, menu_str(_("Use Mouse\tAlt+Ctrl")), NULL, OnSelectEnableMouse, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateEnableMouse, "%Cp", this);
		mi = AG_MenuAction(mt_options, menu_str(_("Keybind...\tAlt+K")), NULL, OnSelectKeybindBox, "%Cp", this);
		AG_MenuSeparator(mt_options);
		mi = AG_MenuAction(mt_options, menu_str(_("Configure...\tAlt+C")), NULL, OnSelectConfigureBox, "%Cp", this);
	}
	mt_help = AG_MenuNode(menu->root, _("Help"), NULL);
	{
		mi = AG_MenuAction(mt_help, menu_str(_("About...")), NULL, OnSelectAbout, "%Cp", this);
	}
}

/**
 * user event
 */
void AG_GUI::ProcessUserEvent(int id)
{
	switch(id) {
//	case EVT_OPEN_LOAD_RECKEY_DIALOG:
//		PlayRecKeyBox(true);
//		break;
	case EVT_OPEN_SAVE_RECKEY_DIALOG:
		RecordRecKeyBox(true);
		break;
	}
}

// create keybind dialog
bool AG_GUI::ShowKeybindDialog(void)
{
	ShowMenu();
	keybindbox->Create();
	return true;
}
// create config dialog
bool AG_GUI::ShowConfigureDialog(void)
{
	ShowMenu();
	configbox->Create();
	return true;
}
// load State
bool AG_GUI::ShowLoadStateDialog(void)
{
	ShowMenu();
	filebox->CreateLoad(AG_FILE_DLG::STATUS, _("Load Status Data")
		, "*.l3r", config.initial_state_path, NULL);
	return true;
}
// save State
bool AG_GUI::ShowSaveStateDialog(void)
{
	ShowMenu();
	filebox->CreateSave(AG_FILE_DLG::STATUS, _("Save Status Data")
		, "*.l3r", config.initial_state_path, NULL);
	return true;
}
// save State and RecordKey
bool AG_GUI::ShowRecordRecKeyDialog(void)
{
	return true;
}
bool AG_GUI::ShowRecordStateAndRecKeyDialog(void)
{
	ShowMenu();
	filebox->CreateSave(AG_FILE_DLG::STATUS_AND_RECKEY, _("Save Status Data")
		, "*.l3r", config.initial_state_path, NULL);
	return true;
}
// Play RecordKey
bool AG_GUI::ShowPlayRecKeyDialog(void)
{
	ShowMenu();
	filebox->CreateLoad(AG_FILE_DLG::RECKEY, _("Play Recorded Keys")
		, "*.l3k", config.initial_state_path, NULL);
	return true;
}
// Record RecordKey
void AG_GUI::RecordRecKeyBox(bool with_status)
{
	ShowMenu();
	filebox->CreateSave(with_status ? AG_FILE_DLG::RECKEY_AFTER_STATUS : AG_FILE_DLG::RECKEY, _("Record Input Keys")
		, "*.l3k", config.initial_state_path, NULL);
}
// show Data Recorder Load Dialog
bool AG_GUI::ShowLoadDataRecDialog(void)
{
	ShowMenu();
	filebox->CreateLoad(AG_FILE_DLG::DATAREC, _("Play Data Recorder Tape")
		, "*.l3,*.l3b,*.l3c,*.wav,*.t9x", config.initial_datarec_path, NULL);
	return true;
}
// show Data Recorder Save Dialog
bool AG_GUI::ShowSaveDataRecDialog(void)
{
	ShowMenu();
	filebox->CreateSave(AG_FILE_DLG::DATAREC, _("Record Data Recorder Tape")
		, "*.l3,*.l3b,*.l3c,*.wav,*.t9x", config.initial_datarec_path, NULL);
	return true;
}

/*
 * Event Handler (static)
 */
// select System Mode
void AG_GUI::OnSelectSystemMode(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);

	gui->ChangeSystemMode(num);
}
// update System Mode
void AG_GUI::OnUpdateSystemMode(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, gui->GetSystemMode() == num);
}
// select FDD Type
void AG_GUI::OnSelectFDDType(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);

	gui->ChangeFddType(num);
}
// update FDD Type
void AG_GUI::OnUpdateFDDType(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, gui->NextFddType() == num);
}
// select sync irq
void AG_GUI::OnSelectSyncIRQ(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->PostEtToggleSyncIRQ();
}
// update sync irq
void AG_GUI::OnUpdateSyncIRQ(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->NowSyncIRQ());
}
// select memory no wait
void AG_GUI::OnSelectMemNoWait(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleMemNoWait();
}
// update memory no wait
void AG_GUI::OnUpdateMemNoWait(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->NowMemNoWait());
}

// Show Status Data Load Dialog
void AG_GUI::OnSelectLoadState(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
//	char *ext = AG_STRING(2);

	gui->ShowLoadStateDialog();
}
// Show Status Data Save Dialog
void AG_GUI::OnSelectSaveState(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
//	char *ext = AG_STRING(2);

	gui->ShowSaveStateDialog();
}

// Show Play RecordKey Dialog
void AG_GUI::OnSelectPlayRecKey(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
//	char *ext = AG_STRING(2);

	gui->ShowPlayRecKeyDialog();
}
// update Play RecordKey
void AG_GUI::OnUpdatePlayRecKey(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->NowPlayingRecKey());
}
// stop Play RecordKey
void AG_GUI::OnSelectStopPlayRecKey(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->StopPlayRecKey();
}

// Show Record RecordKey Dialog
void AG_GUI::OnSelectRecordRecKey(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
//	char *ext = AG_STRING(2);

	gui->ShowRecordStateAndRecKeyDialog();
}
// update Record RecordKey
void AG_GUI::OnUpdateRecordRecKey(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->NowRecordingRecKey());
}
// stop Play RecordKey
void AG_GUI::OnSelectStopRecordRecKey(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->StopRecordRecKey();
}

// update Rewind Data Recorder file
void AG_GUI::OnSelectRewindDataRec(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->PostEtRewindDataRecMessage();
}
// update Fast-Forward Data Recorder file
void AG_GUI::OnSelectFastForwardDataRec(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->PostEtFastForwardDataRecMessage();
}
// update Realmode Data Recorder
void AG_GUI::OnSelectRealModeDataRec(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->PostEtToggleRealModeDataRecMessage();
}
// update Realmode Data Recorder
void AG_GUI::OnUpdateRealModeDataRec(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, gui->NowRealModeDataRec());
}

// Show Floppy Disk Change Side
void AG_GUI::OnSelectChangeSideFloppyDisk(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int   drv = AG_INT(2);

	gui->PostEtChangeSideFloppyDisk(drv);
}
// update Floppy Disk Change Side
void AG_GUI::OnUpdateChangeSideFloppyDisk(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int   drv = AG_INT(2);

	int side = gui->GetSideFloppyDisk(drv);

	if ((mi) && (mi)->text) {
		if (side) {
			char *p = strchr((mi)->text, 'B');
			// ..Change side to A
			if (p != NULL) *p = 'A';
		} else {
			char *p = strchr((mi)->text, 'A');
			// ..Change side to B
			if (p != NULL) *p = 'B';
		}
	}
	AG_GUI_MENU_ENABLE(mi, gui->GetFddType() != 0);
}

// Show Floppy Disk Write Protect
void AG_GUI::OnSelectWriteProtectFloppyDisk(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int   drv = AG_INT(2);

	gui->PostEtToggleWriteProtectFloppyDisk(drv);
}
// update Floppy Disk Write Protect
void AG_GUI::OnUpdateWriteProtectFloppyDisk(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int   drv = AG_INT(2);

	AG_GUI_MENU_ENABLE(mi, gui->InsertedFloppyDisk(drv));
	AG_GUI_MENU_CHECK(mi, gui->WriteProtectedFloppyDisk(drv));
}

// Change Stretch screen
void AG_GUI::OnSelectStretchScreen(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->ToggleStretchScreen();
}
// update StretchScreen
void AG_GUI::OnUpdateStretchScreen(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, gui->NowStretchedScreen());
}

// Change Cutout screen
void AG_GUI::OnSelectCutoutScreen(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->ToggleCutoutScreen();
}
// update CutoutScreen
void AG_GUI::OnUpdateCutoutScreen(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, gui->NowCutoutedScreen());
}

// Change After Image
void AG_GUI::OnSelectAfterImage(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);
	gui->PostEtChangeAfterImage(num);
}
// update After Image
void AG_GUI::OnUpdateAfterImage(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, gui->GetAfterImageMode() == num);
}

// Change RGB Type
void AG_GUI::OnSelectRGBType(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);
	gui->ChangeRGBType(num);
}
// update RGB Type
void AG_GUI::OnUpdateRGBType(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, gui->GetRGBTypeMode() == num);
}
#ifdef USE_OPENGL
// Change Use OpenGL
void AG_GUI::OnSelectUseOpenGL(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);
	gui->ChangeUseOpenGL(num);
}
// update Use OpenGL
void AG_GUI::OnUpdateUseOpenGL(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, gui->GetOpenGLMode() == num);
}
#endif

// Change Ledbox
void AG_GUI::OnSelectLedBox(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleLedBox();
}
// update Ledbox
void AG_GUI::OnUpdateLedBox(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->IsShownLedBox());
}
// Change MsgBoard
void AG_GUI::OnSelectMsgBoard(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleMessageBoard();
}
// update MsgBoard
void AG_GUI::OnUpdateMsgBoard(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->IsShownMessageBoard());
}

// Change UseJoypad
void AG_GUI::OnSelectUseJoypad(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleUseJoypad();
}
// update UseJoypad
void AG_GUI::OnUpdateUseJoypad(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->IsEnableJoypad());
}

// Change EnableLightpen
void AG_GUI::OnSelectEnableLightpen(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleEnableLightpen();
}
// update EnableLightpen
void AG_GUI::OnUpdateEnableLightpen(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->IsEnableLightpen());
}

// Change EnableMouse
void AG_GUI::OnSelectEnableMouse(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleUseMouse();
}
// update EnableMouse
void AG_GUI::OnUpdateEnableMouse(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->IsEnableMouse());
}

// create keybind dialog
void AG_GUI::OnSelectKeybindBox(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ShowKeybindDialog();
}

// create config dialog
void AG_GUI::OnSelectConfigureBox(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ShowConfigureDialog();
}

// Change Print Printer
void AG_GUI::OnSelectPrintPrinter(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	gui->PostEtPrintPrinterMessage(drv);
}
// update Print Printer
void AG_GUI::OnUpdatePrintPrinter(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	int size = gui->GetPrinterBufferSize(drv);

	AG_GUI_MENU_ENABLE(mi, size > 0);
}

// Change Direct Printer
void AG_GUI::OnSelectDirectPrinter(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	gui->PostEtEnablePrinterDirectMessage(drv);
}
// update Direct Printer
void AG_GUI::OnUpdateDirectPrinter(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, gui->IsEnablePrinterDirect(drv));
}

// Change Comm Server
void AG_GUI::OnSelectCommServer(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	gui->PostEtEnableCommServerMessage(drv);
}
// update Comm Server
void AG_GUI::OnUpdateCommServer(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, gui->IsEnableCommServer(drv));
}

// Change Comm connect
void AG_GUI::OnSelectCommConnect(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	gui->PostEtConnectCommMessage(drv);
}
// update Comm connect
void AG_GUI::OnUpdateCommConnect(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, gui->NowConnectingComm(drv));
}

}; /* namespace GUI_AGAR */
