/** @file cocoa_basepanel.h

 HITACHI MB-S1 Emulator 'EmuB-S1'
 Skelton for retropc emulator
 SDL edition + Cocoa GUI

 @author Sasaji
 @date   2015.04.30 -

 @brief [ panel ]
 */

#ifndef _COCOA_BASEPANEL_H_
#define _COCOA_BASEPANEL_H_

#import <Cocoa/Cocoa.h>

@interface CocoaLabel : NSTextField
+ (CocoaLabel *)create:(NSRect)re title:(const char *)title;
+ (CocoaLabel *)create:(NSRect)re title:(const char *)title align:(NSTextAlignment)align;
@end

@interface CocoaTextField : NSTextField
+ (CocoaTextField *)create:(NSRect)re text:(const char *)text action:(SEL)action;
+ (CocoaTextField *)create:(NSRect)re text:(const char *)text action:(SEL)action align:(NSTextAlignment)align;
+ (CocoaTextField *)create:(NSRect)re num:(int)num action:(SEL)action;
+ (CocoaTextField *)create:(NSRect)re num:(int)num action:(SEL)action align:(NSTextAlignment)align;
@end

@interface CocoaButton : NSButton
{
	id relatedObject;
}
@property (assign) id relatedObject;
+ (CocoaButton *)create:(NSRect)re title:(const char *)title action:(SEL)action;
@end

@interface CocoaPopUpButton : NSPopUpButton
+ (CocoaPopUpButton *)create:(NSRect)re items:(const char **)items action:(SEL)action selidx:(int)selidx;
@end

@interface CocoaCheckBox : NSButton
{
	int index;
}
@property (nonatomic) int index;
+ (CocoaCheckBox *)create:(NSRect)re title:(const char *)title action:(SEL)action value:(bool)value;
+ (CocoaCheckBox *)create:(NSRect)re title:(const char *)title index:(int)index action:(SEL)action value:(bool)value;
@end

@interface CocoaRadioGroup : NSMatrix
+ (CocoaRadioGroup *)create:(NSRect)re rows:(int)rows cols:(int)cols titles:(const char **)titles action:(SEL)action selidx:(int)selidx;
@end

@interface CocoaSlider : NSSlider
{
	int index;
}
@property (nonatomic) int index;
+ (CocoaSlider *)create:(NSRect)re action:(SEL)action value:(int)value;
+ (CocoaSlider *)create:(NSRect)re index:(int)index action:(SEL)action value:(int)value;
@end

@interface CocoaTabView : NSTabView
+ (CocoaTabView *)create:(NSRect)re;
+ (CocoaTabView *)create:(NSRect)re tabs:(const char **)tabs;
- (NSTabViewItem *)addTabItem:(const char *)label;
- (NSRect)adjustFrameSize:(NSRect)re;
@end

@interface CocoaBox : NSBox
+ (CocoaBox *)create:(NSRect)re title:(const char *)title;
- (NSRect)adjustFrameSize:(NSRect)re;
@end

@interface CocoaView : NSView
{
	int max_right;
	int max_bottom;

	int margin;
}
@property int max_right;
@property int max_bottom;
@property int margin;
- (id)init;
- (BOOL)isFlipped;
- (NSRect)makeRect:(int)left :(int)top :(int)w :(int)h;
- (NSRect)remakeSize:(NSRect)re :(int)w :(int)h;
- (NSRect)remakePosition:(NSRect)re :(int)left :(int)top :(int)new_w :(int)new_h;
- (NSRect)remakeLeft:(NSRect)re :(int)left :(int)new_w :(int)new_h;
- (NSRect)remakeLeft:(NSRect)re :(int)left;
- (NSRect)remakeTop:(NSRect)re :(int)top :(int)new_w :(int)new_h;
- (NSRect)remakeTop:(NSRect)re :(int)top;
- (NSRect)addPosition:(NSRect)re :(int)offsetx :(int)offsety :(int)new_w :(int)new_h;
- (NSRect)addWidth:(NSRect)re  :(int)new_w :(int)new_h;
- (NSRect)addHeight:(NSRect)re :(int)new_w :(int)new_h;
+ (int)topPos:(NSRect)re;
+ (int)bottomPos:(NSRect)re;
- (void)adjustFrameSize;
//- (NSRect)adjustFrameSize:(NSRect)re;
@end

@interface CocoaBasePanel : NSPanel
- (id)init;
- (int)margin;
- (void)adjustFrame;
@end

#endif /* _COCOA_BASEPANEL_H_ */
