/** @file ledbox.cpp

	HITACHI MB-S1 Emulator 'EmuB-S1'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.03.08 -

	@brief [ led box ]
*/
#include "ledbox.h"

#define RECT_IN(st, xx, yy, ww, hh) { \
	st.x = xx; \
	st.y = yy; \
	st.w = ww; \
	st.h = hh; \
}

LedBox::LedBox()
{
	surface = NULL;

	suf_base = NULL;

	suf_kata = NULL;
	suf_hira = NULL;
	suf_caps = NULL;

	suf_play = NULL;
	suf_rec  = NULL;

	suf_digit10 = NULL;

#ifdef USE_FD1
	suf_fddr = NULL;
	suf_fddg = NULL;
#endif

	memset(&win_pt, 0, sizeof(win_pt));
	RECT_IN(parent_pt,0,0,0,0);

	flag_old = -1;

	enable = false;
}

LedBox::~LedBox()
{
#ifdef USE_FD1
	if (suf_fddr != NULL) SDL_FreeSurface(suf_fddr);
	if (suf_fddg != NULL) SDL_FreeSurface(suf_fddg);
#endif
	if (suf_digit10 != NULL) SDL_FreeSurface(suf_digit10);

	if (suf_play != NULL) SDL_FreeSurface(suf_play);
	if (suf_rec  != NULL) SDL_FreeSurface(suf_rec);

	if (suf_kata != NULL) SDL_FreeSurface(suf_kata);
	if (suf_hira != NULL) SDL_FreeSurface(suf_hira);
	if (suf_caps != NULL) SDL_FreeSurface(suf_caps);

	if (suf_base != NULL) SDL_FreeSurface(suf_base);

	if (surface  != NULL) SDL_FreeSurface(surface);
}

bool LedBox::Create(char *res_path, SDL_PixelFormat *format)
{
	char path[_MAX_PATH];

	enable = false;

	sprintf(path,"%s%s",res_path,"ledbox_8.bmp");
	suf_base = SDL_LoadBMP(path);
	if (suf_base == NULL) return false;

	sprintf(path,"%s%s",res_path,"led_r_h_red.bmp");
	suf_kata = SDL_LoadBMP(path);
	if (suf_kata == NULL) return false;
	RECT_IN(pt_kata, 30, 3, suf_kata->w, suf_kata->h);
	sprintf(path,"%s%s",res_path,"led_r_h_green.bmp");
	suf_hira = SDL_LoadBMP(path);
	if (suf_hira == NULL) return false;
	RECT_IN(pt_hira, 30, 11, suf_hira->w, suf_hira->h);
	sprintf(path,"%s%s",res_path,"led_rv_h_red.bmp");
	suf_caps = SDL_LoadBMP(path);
	if (suf_caps == NULL) return false;
	RECT_IN(pt_caps, 3, 3, suf_caps->w, suf_caps->h);

	sprintf(path,"%s%s",res_path,"led_rr_h_green.bmp");
	suf_play = SDL_LoadBMP(path);
	if (suf_play == NULL) return false;
	RECT_IN(pt_play, 65, 4, suf_play->w, suf_play->h);
	sprintf(path,"%s%s",res_path,"led_c_h_red.bmp");
	suf_rec  = SDL_LoadBMP(path);
	if (suf_rec == NULL) return false;
	RECT_IN(pt_rec,  98, 4, suf_rec->w, suf_rec->h);

	sprintf(path,"%s%s",res_path,"digit10.bmp");
	suf_digit10 = SDL_LoadBMP(path);
	if (suf_digit10 == NULL) return false;
	RECT_IN(pt_digit1, 147, 4, 8, 12);
	RECT_IN(pt_digit2, 138, 4, 8, 12);
	RECT_IN(pt_digit3, 129, 4, 8, 12);

#ifdef USE_FD1
	sprintf(path,"%s%s",res_path,"led_rv_h_red.bmp");
	suf_fddr = SDL_LoadBMP(path);
	if (suf_fddr == NULL) return false;
	sprintf(path,"%s%s",res_path,"led_rv_h_green.bmp");
	suf_fddg = SDL_LoadBMP(path);
	if (suf_fddg == NULL) return false;
	RECT_IN(pt_fdd1, 162, 3, suf_fddr->w, suf_fddr->h);
	RECT_IN(pt_fdd2, 187, 3, suf_fddr->w, suf_fddr->h);
	RECT_IN(pt_fdd3, 211, 3, suf_fddr->w, suf_fddr->h);
	RECT_IN(pt_fdd4, 236, 3, suf_fddr->w, suf_fddr->h);
#endif

	if (!CreateSurface(format)) {
		return false;
	}

	parent_pt.w = surface->w;
	parent_pt.h = surface->h;

	enable = true;
	return true;
}

bool LedBox::CreateSurface(SDL_PixelFormat *format)
{
	if (suf_base == NULL) return false;
	if (surface != NULL) {
		SDL_FreeSurface(surface);
	}
	surface = SDL_ConvertSurface(suf_base, format, SDL_SWSURFACE);
	if (surface == NULL) return false;

	return true;
}

void LedBox::SetPos(int left, int top, int right, int bottom, int place)
{
	win_pt.left = left;
	win_pt.top = top;
	win_pt.right = right;
	win_pt.bottom = bottom;
	win_pt.place = place;

	if (place & 1) {
		// base right
		parent_pt.x = right - parent_pt.w;
	} else {
		// base left
		parent_pt.x = left;
	}
	if (place & 2) {
		// base bottom
		parent_pt.y = bottom - parent_pt.h;
	} else {
		// base top
		parent_pt.y = top;
	}
}
void LedBox::SetPos(int place)
{
	SetPos(win_pt.left, win_pt.top, win_pt.right, win_pt.bottom, place);
}

void LedBox::MixScreen(SDL_Surface *screen, int x, int y)
{
	SDL_Rect re = { static_cast<Sint16>(x), static_cast<Sint16>(y), parent_pt.w, parent_pt.h };

	SDL_BlitSurface(surface, NULL, screen, &re);
}
void LedBox::MixScreen(SDL_Surface *screen)
{
	MixScreen(screen, parent_pt.x, parent_pt.y);
}

void LedBox::Update(SDL_Surface *screen, int flag, int x, int y)
{
	if (flag != flag_old) {
		SDL_BlitSurface(suf_base, NULL, surface, NULL);

		// katakana
		if (flag & 0x0001) {
			SDL_BlitSurface(suf_kata, NULL, surface, &pt_kata);
		}
		// hiragana
		if (flag & 0x0002) {
			SDL_BlitSurface(suf_hira, NULL, surface, &pt_hira);
		}
		// caps
		if ((flag & 0x0004) == 0) {
			SDL_BlitSurface(suf_caps, NULL, surface, &pt_caps);
		}
		// play
		if (flag & 0x0010) {
			SDL_BlitSurface(suf_play, NULL, surface, &pt_play);
		}
		// rec
		if (flag & 0x0020) {
			SDL_BlitSurface(suf_rec , NULL, surface, &pt_rec);
		}
#ifdef USE_FD1
		// fdd1
		if ((flag & 0x1100) == 0x0100) {
			SDL_BlitSurface(suf_fddg, NULL, surface, &pt_fdd1);
		} else if ((flag & 0x1100) == 0x1100) {
			SDL_BlitSurface(suf_fddr, NULL, surface, &pt_fdd1);
		}
#endif
#ifdef USE_FD2
		// fdd2
		if ((flag & 0x2200) == 0x0200) {
			SDL_BlitSurface(suf_fddg, NULL, surface, &pt_fdd2);
		} else if ((flag & 0x2200) == 0x2200) {
			SDL_BlitSurface(suf_fddr, NULL, surface, &pt_fdd2);
		}
#endif
#ifdef USE_FD3
		// fdd3
		if ((flag & 0x4400) == 0x0400) {
			SDL_BlitSurface(suf_fddg, NULL, surface, &pt_fdd3);
		} else if ((flag & 0x4400) == 0x4400) {
			SDL_BlitSurface(suf_fddr, NULL, surface, &pt_fdd3);
		}
#endif
#ifdef USE_FD4
		// fdd4
		if ((flag & 0x8800) == 0x0800) {
			SDL_BlitSurface(suf_fddg, NULL, surface, &pt_fdd4);
		} else if ((flag & 0x8800) == 0x8800) {
			SDL_BlitSurface(suf_fddr, NULL, surface, &pt_fdd4);
		}
#endif
		// cmt counter
		uint32 count = (flag >> 16);
		int i = (count / 12);
		int f = (count % 12);
		SDL_Rect re;
		RECT_IN(re, 0, 0, pt_digit1.w, pt_digit1.h);
		re.y = (i % 10) * pt_digit1.h + f;
		SDL_BlitSurface(suf_digit10, &re, surface, &pt_digit1);
		re.y = ((i / 10) % 10) * pt_digit2.h + ((i % 10) == 9 ? f : 0);
		SDL_BlitSurface(suf_digit10, &re, surface, &pt_digit2);
		re.y = ((i / 100) % 10) * pt_digit3.h + ((i % 100) == 99 ? f : 0);
		SDL_BlitSurface(suf_digit10, &re, surface, &pt_digit3);


	}
	flag_old = flag;

	MixScreen(screen, x, y);
}
void LedBox::Update(SDL_Surface *screen, int flag)
{
	Update(screen, flag, parent_pt.x, parent_pt.y);
}