/** @file ym2203.h

	Skelton for retropc emulator

	@author Takeda.Toshiya
	@date   2006.09.15-

	@note
	Modified by Sasaji at 2012.06.20

	@brief [ AY-3-8912 / YM2203 / YM2608 ]
*/

#ifndef _YM2203_H_
#define _YM2203_H_

#include "vm.h"
#include "../emu.h"
#include "device.h"
#include "fmgen/opna.h"

#define SIG_YM2203_PORT_A	0
#define SIG_YM2203_PORT_B	1
#define SIG_YM2203_MUTE		2

/**
	@brief AY-3-8912 / YM2203 / YM2608 - FM Sound Operator
*/
class YM2203 : public DEVICE
{
private:
#ifdef HAS_YM2608
	FM::OPNA* chip;
#else
	FM::OPN* chip;
#endif

	uint8 ch, mode;
#ifdef HAS_YM2608
	uint8 ch1, data1;
#endif

	typedef struct {
		uint8 wreg;
		uint8 rreg;
		bool first;
		// output signals
		outputs_t outputs;
	} port_t;
	port_t port[2];

	bool mute;
	int usec_per_vline;
#ifdef USE_EMU_INHERENT_SPEC
	int sound_volume;
	int32 buffer_tmp[40];
#endif

#ifndef HAS_AY_3_8912
	// output signals
	outputs_t outputs_irq;
	void update_interrupt();
#endif

#ifdef USE_EMU_INHERENT_SPEC
	//for resume
#pragma pack(1)
	struct vm_state_st {
		uint8 ch, mode;
		// PSG register only
		uint8 reg[13];
		struct {
			uint8 wreg;
			uint8 rreg;
			uint8 first;
		} port[2];

		char reserved[11];
	};
#pragma pack()
#endif

public:
	YM2203(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier) {
		set_class_name("YM2203");
		for(int i = 0; i < 2; i++) {
			init_output_signals(&port[i].outputs);
			port[i].wreg = port[i].rreg = 0;//0xff;
		}
#ifndef HAS_AY_3_8912
		init_output_signals(&outputs_irq);
#endif
	}
	~YM2203() {}

	// common functions
	void initialize();
	void release();
	void reset();
	void write_io8(uint32 addr, uint32 data);
	uint32 read_io8(uint32 addr);
	void write_signal(int id, uint32 data, uint32 mask);
	void event_vline(int v, int clock);
	void mix(int32* buffer, int cnt);
	void update_timing(int new_clocks, double new_frames_per_sec, int new_lines_per_frame);
#ifdef USE_EMU_INHERENT_SPEC
	void set_volume(int volume);
#endif

	// unique functions
#ifndef HAS_AY_3_8912
	void set_context_irq(DEVICE* device, int id, uint32 mask) {
		register_output_signal(&outputs_irq, device, id, mask);
	}
#endif
	void set_context_port_a(DEVICE* device, int id, uint32 mask, int shift) {
		register_output_signal(&port[0].outputs, device, id, mask, shift);
	}
	void set_context_port_b(DEVICE* device, int id, uint32 mask, int shift) {
		register_output_signal(&port[1].outputs, device, id, mask, shift);
	}
	void init(int rate, int clock, int samples, int volf, int volp);

#ifdef USE_EMU_INHERENT_SPEC
	void save_state(FILEIO *fp);
	void load_state(FILEIO *fp);
#endif
};

#endif

